@@ -7,17 +7,19 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#include "config.h"
+
 #include <sys/prctl.h>
+#include <sys/syscall.h>
+#include <unistd.h>
 
 #include <atomic>
 #include <cassert>
+#include <ctime>
 #include <iomanip>
 #include <sstream>
 #include <string>
 
-#include <sys/syscall.h>
-#include <unistd.h>
-
 #include <Poco/ConsoleChannel.h>
 #include <Poco/DateTimeFormatter.h>
 #include <Poco/FileChannel.h>
@@ -29,6 +31,7 @@
 #include <Poco/Timestamp.h>
 
 #include "Log.hpp"
+#include "Util.hpp"
 
 static char LogPrefix[256] = { '\0' };
 
@@ -75,39 +78,54 @@ namespace Log
         }
     }
 
-    static void getPrefix(char *buffer, const char* level)
+    // We need a signal safe means of writing messages
+    //   $ man 7 signal
+    void signalLogNumber(size_t num)
     {
-        // FIXME: If running under systemd it is redundant to output timestamps, as those will be
-        // attached to messages that the systemd journalling mechanism picks up anyway, won't they?
-
-        std::string time = DateTimeFormatter::format(Poco::Timestamp(), "%H:%M:%s");
-
-        char procName[32]; // we really need only 16
-        if (prctl(PR_GET_NAME, reinterpret_cast<unsigned long>(procName), 0, 0, 0) != 0)
+        int i;
+        char buf[22];
+        buf[21] = '\0';
+        for (i = 20; i > 0 && num > 0; --i)
         {
-            strncpy(procName, "<noid>", sizeof(procName) - 1);
+            buf[i] = '0' + num % 10;
+            num /= 10;
         }
-
-        const char* appName = (Source.inited ? Source.id.c_str() : "<shutdown>");
-        assert(strlen(appName) + 32 + 28 < 1024 - 1);
-
-        snprintf(buffer, 4095, "%s-%.04lu %s [ %s ] %s  ", appName,
-                 syscall(SYS_gettid),
-                 time.c_str(),
-                 procName, level);
+        signalLog(buf + i + 1);
     }
 
-    std::string prefix(const char* level)
+    char* prefix(char* buffer, const char* level, bool sigSafe)
     {
-        char buffer[1024];
-        getPrefix(buffer, level);
-        return std::string(buffer);
+        long osTid;
+        char procName[32];
+        const char *threadName = procName;
+        if (sigSafe)
+        {
+            osTid = syscall(SYS_gettid);
+
+            if (prctl(PR_GET_NAME, reinterpret_cast<unsigned long>(procName), 0, 0, 0) != 0)
+                strncpy(procName, "<noid>", sizeof(procName) - 1);
+        }
+        else
+        {
+            osTid = Util::getThreadId();
+            threadName = Util::getThreadName();
+        }
+
+        Poco::DateTime time;
+        snprintf(buffer, 1023, "%s-%.05lu %.4u-%.2u-%.2u %.2u:%.2u:%.2u.%.6u [ %s ] %s  ",
+                    (Source.inited ? Source.id.c_str() : "<shutdown>"),
+                    osTid,
+                    time.year(), time.month(), time.day(),
+                    time.hour(), time.minute(), time.second(),
+                    time.millisecond() * 1000 + time.microsecond(),
+                    threadName, level);
+        return buffer;
     }
 
     void signalLogPrefix()
     {
         char buffer[1024];
-        getPrefix(buffer, "SIG");
+        prefix(buffer, "SIG", true);
         signalLog(buffer);
     }
 
@@ -141,12 +159,31 @@ namespace Log
         else
             channel = static_cast<Poco::Channel*>(new Poco::ConsoleChannel());
 
+        /**
+         * Open the channel explicitly, instead of waiting for first log message
+         * This is important especially for the kit process where opening the channel
+         * after chroot can cause file creation inside the jail instead of outside
+         * */
+        channel->open();
         auto& logger = Poco::Logger::create(Source.name, channel, Poco::Message::PRIO_TRACE);
 
         logger.setLevel(logLevel.empty() ? std::string("trace") : logLevel);
 
-        info("Initializing " + name);
-        info("Log level is [" + std::to_string(logger.getLevel()) + "].");
+        const std::time_t t = std::time(nullptr);
+        oss.str("");
+        oss.clear();
+
+        oss << "Initializing " << name << ".";
+
+        // TODO: replace with std::put_time when we move to gcc 5+.
+        char buf[32];
+        if (strftime(buf, sizeof(buf), "%a %F %T%z", std::localtime(&t)) > 0)
+        {
+            oss << " Local time: " << buf << ".";
+        }
+
+        oss <<  " Log level is [" << logger.getLevel() << "].";
+        info(oss.str());
     }
 
     Poco::Logger& logger()
@@ -156,42 +193,42 @@ namespace Log
 
     void trace(const std::string& msg)
     {
-        logger().trace(prefix("TRC") + msg);
+        LOG_TRC(msg);
     }
 
     void debug(const std::string& msg)
     {
-        logger().debug(prefix("DBG") + msg);
+        LOG_DBG(msg);
     }
 
     void info(const std::string& msg)
     {
-        logger().information(prefix("INF") + msg);
+        LOG_INF(msg);
     }
 
     void warn(const std::string& msg)
     {
-        logger().warning(prefix("WRN") + msg);
+        LOG_WRN(msg);
     }
 
     void error(const std::string& msg)
     {
-        logger().error(prefix("ERR") + msg);
+        LOG_ERR(msg);
     }
 
     void syserror(const std::string& msg)
     {
-        logger().error(prefix("ERR") + msg + " (errno: " + std::string(std::strerror(errno)) + ")");
+        LOG_SYS(msg);
     }
 
     void fatal(const std::string& msg)
     {
-        logger().fatal(prefix("FTL") + msg);
+        LOG_FTL(msg);
     }
 
     void sysfatal(const std::string& msg)
     {
-        logger().fatal(prefix("FTL") + msg + " (errno: " + std::string(std::strerror(errno)) + ")");
+        LOG_SFL(msg);
     }
 }
 
