@@ -20,6 +20,8 @@
 #include <unistd.h>
 #include <utime.h>
 #include <sys/sysmacros.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 
 #include <atomic>
 #include <cassert>
@@ -104,6 +106,12 @@ class Document;
 static std::shared_ptr<Document> document;
 static LokHookFunction2* initFunction = nullptr;
 
+#if ENABLE_DEBUG
+#  define ADD_DEBUG_RENDERID(s) ((s)+ " renderid=" + Util::UniqueId())
+#else
+#  define ADD_DEBUG_RENDERID(s) (s)
+#endif
+
 namespace
 {
 #ifndef BUILDING_TESTS
@@ -153,7 +161,7 @@ namespace
             File(newPath.parent()).createDirectories();
             if (link(fpath, newPath.toString().c_str()) == -1)
             {
-                LOG_INF("link(\"" << fpath << "\", \"" <<
+                LOG_SYS("link(\"" << fpath << "\", \"" <<
                         newPath.toString() << "\") failed. Will copy.");
                 try
                 {
@@ -283,11 +291,14 @@ namespace
 class PngCache
 {
     typedef std::shared_ptr< std::vector< char > > CacheData;
+
     struct CacheEntry {
         size_t    _hitCount;
+        TileWireId _wireId;
         CacheData _data;
-        CacheEntry(size_t defaultSize) :
+        CacheEntry(size_t defaultSize, TileWireId id) :
             _hitCount(1),   // Every entry is used at least once; prevent removal at birth.
+            _wireId(id),
             _data( new std::vector< char >() )
         {
             _data->reserve( defaultSize );
@@ -298,7 +309,32 @@ class PngCache
     static const size_t CacheSizeHardLimit = CacheSizeSoftLimit * 2;
     size_t _cacheHits;
     size_t _cacheTests;
-    std::map< uint64_t, CacheEntry > _cache;
+    TileWireId _nextId;
+
+    std::map< TileBinaryHash, CacheEntry > _cache;
+    std::map< TileWireId, TileBinaryHash > _wireToHash;
+
+    void clearCache(bool logStats = false)
+    {
+        if (logStats)
+            LOG_DBG("cache clear " << _cache.size() << " items total size " <<
+                    _cacheSize << " current hits " << _cacheHits);
+        _cache.clear();
+        _cacheSize = 0;
+        _cacheHits = 0;
+        _cacheTests = 0;
+        _nextId = 1;
+    }
+
+    // Keep these ids small and wrap them.
+    TileWireId createNewWireId()
+    {
+        TileWireId id = ++_nextId;
+        // FIXME: if we wrap - we should flush the clients too really ...
+        if (id < 1)
+            clearCache(true);
+        return id;
+    }
 
     void balanceCache()
     {
@@ -323,6 +359,11 @@ class PngCache
                     // Shrink cache when we exceed the size to maximize
                     // the chance of hitting these entries in the future.
                     _cacheSize -= it->second._data->size();
+
+                    auto wIt = _wireToHash.find(it->second._wireId);
+                    assert(wIt != _wireToHash.end());
+                    _wireToHash.erase(wIt);
+
                     it = _cache.erase(it);
                 }
                 else
@@ -365,10 +406,10 @@ class PngCache
                                    int width, int height,
                                    int bufferWidth, int bufferHeight,
                                    std::vector<char>& output, LibreOfficeKitTileMode mode,
-                                   const uint64_t hash)
+                                   TileBinaryHash hash, TileWireId wid, TileWireId /* oldWid */)
     {
         LOG_DBG("PNG cache with hash " << hash << " missed.");
-        CacheEntry newEntry(bufferWidth * bufferHeight * 1);
+        CacheEntry newEntry(bufferWidth * bufferHeight * 1, wid);
         if (Png::encodeSubBufferToPNG(pixmap, startX, startY, width, height,
                                       bufferWidth, bufferHeight,
                                       *newEntry._data, mode))
@@ -391,42 +432,167 @@ class PngCache
     }
 
 public:
-    PngCache() :
-        _cacheSize(0),
-        _cacheHits(0),
-        _cacheTests(0)
+    PngCache()
     {
+        clearCache();
+    }
+
+    TileWireId hashToWireId(TileBinaryHash id)
+    {
+        TileWireId wid;
+        if (id == 0)
+            return 0;
+        auto it = _cache.find(id);
+        if (it != _cache.end())
+            wid = it->second._wireId;
+        else
+        {
+            wid = createNewWireId();
+            _wireToHash.emplace(wid, id);
+        }
+        return wid;
     }
 
     bool encodeBufferToPNG(unsigned char* pixmap, int width, int height,
                            std::vector<char>& output, LibreOfficeKitTileMode mode,
-                           uint64_t hash)
+                           TileBinaryHash hash, TileWireId wid, TileWireId oldWid)
     {
         if (cacheTest(hash, output))
-        {
             return true;
-        }
 
         return cacheEncodeSubBufferToPNG(pixmap, 0, 0, width, height,
-                                         width, height, output, mode, hash);
+                                         width, height, output, mode,
+                                         hash, wid, oldWid);
     }
 
     bool encodeSubBufferToPNG(unsigned char* pixmap, size_t startX, size_t startY,
                               int width, int height,
                               int bufferWidth, int bufferHeight,
                               std::vector<char>& output, LibreOfficeKitTileMode mode,
-                              uint64_t hash)
+                              TileBinaryHash hash, TileWireId wid, TileWireId oldWid)
     {
         if (cacheTest(hash, output))
-        {
             return true;
-        }
 
         return cacheEncodeSubBufferToPNG(pixmap, startX, startY, width, height,
-                                         bufferWidth, bufferHeight, output, mode, hash);
+                                         bufferWidth, bufferHeight, output, mode,
+                                         hash, wid, oldWid);
     }
 };
 
+class Watermark
+{
+public:
+    Watermark(const std::shared_ptr<lok::Document>& loKitDoc, const std::string& text)
+        : _loKitDoc(loKitDoc)
+        , _text(text)
+        , _font("Liberation Sans")
+        , _width(0)
+        , _height(0)
+        , _color{64, 64, 64}
+        , _alphaLevel(0.2)
+        , _pixmap(nullptr)
+    {
+    }
+
+    ~Watermark()
+    {
+        if (_pixmap)
+            std::free(_pixmap);
+    }
+
+    void blending(unsigned char* tilePixmap,
+                   int offsetX, int offsetY,
+                   int tilesPixmapWidth, int tilesPixmapHeight,
+                   int tileWidth, int tileHeight,
+                   LibreOfficeKitTileMode mode)
+    {
+        // set requested watermark size a little bit smaller than tile size
+        int width = tileWidth * 0.9;
+        int height = tileHeight * 0.9;
+
+        const unsigned char* pixmap = getPixmap(width, height);
+
+        if (pixmap && tilePixmap)
+        {
+            unsigned int pixmapSize = tilesPixmapWidth * tilesPixmapHeight * 4;
+            int maxX = std::min(tileWidth, _width);
+            int maxY = std::min(tileHeight, _height);
+
+            // center watermark
+            offsetX += (tileWidth - maxX) / 2;
+            offsetY += (tileHeight - maxY) / 2;
+
+            for (int y = 0; y < maxY; ++y)
+            {
+                for (int x = 0; x < maxX; ++x)
+                {
+                    unsigned int i = (y * _width + x) * 4;
+                    unsigned int alpha = pixmap[i + 3];
+                    if (alpha)
+                    {
+                        for (int h = 0; h < 3; ++h)
+                        {
+                            unsigned int j = ((y + offsetY) * tilesPixmapWidth  + (x + offsetX)) * 4 + h;
+                            if (j < pixmapSize)
+                            {
+                                unsigned int color = (mode == LOK_TILEMODE_BGRA) ? _color[2 - h] : _color[h];
+
+                                // original alpha blending for smoothing text edges
+                                color = ((color * alpha) + tilePixmap[j] * (255 - alpha)) / 255;
+                                // blending between document tile and watermark
+                                tilePixmap[j] = color * _alphaLevel + tilePixmap[j] * (1 - _alphaLevel);
+                           }
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+private:
+    const unsigned char* getPixmap(int width, int height)
+    {
+        if (_pixmap && width == _width && height == _height)
+            return _pixmap;
+
+        if (_pixmap)
+            std::free(_pixmap);
+
+        _width = width;
+        _height = height;
+
+        if (!_loKitDoc)
+        {
+            LOG_ERR("Watermark rendering requested without a valid document.");
+            return nullptr;
+        }
+
+        // renderFont returns a buffer based on RGBA mode, where r, g, b
+        // are always set to 0 (black) and the alpha level is 0 everywhere
+        // except on the text area; the alpha level take into account of
+        // performing anti-aliasing over the text edges.
+        _pixmap = _loKitDoc->renderFont(_font.c_str(), _text.c_str(), &_width, &_height);
+
+        if (!_pixmap)
+        {
+            LOG_ERR("Watermark: rendering failed.");
+        }
+
+        return _pixmap;
+    }
+
+private:
+    std::shared_ptr<lok::Document> _loKitDoc;
+    std::string _text;
+    std::string _font;
+    int _width;
+    int _height;
+    unsigned char _color[3];
+    double _alphaLevel;
+    unsigned char* _pixmap;
+};
+
 static FILE* ProcSMapsFile = nullptr;
 
 /// A document container.
@@ -464,7 +630,9 @@ public:
         _isDocPasswordProtected(false),
         _docPasswordType(PasswordType::ToView),
         _stop(false),
-        _isLoading(0)
+        _isLoading(0),
+        _editorId(-1),
+        _editorChangeWarning(false)
     {
         LOG_INF("Document ctor for [" << _docKey <<
                 "] url [" << _url << "] on child [" << _jailId <<
@@ -509,6 +677,10 @@ public:
             auto session = std::make_shared<ChildSession>(sessionId, _jailId, *this);
             _sessions.emplace(sessionId, session);
 
+            int viewId = session->getViewId();
+            _lastUpdatedAt[viewId] = std::chrono::steady_clock::now();
+            _speedCount[viewId] = 0;
+
             LOG_DBG("Sessions: " << _sessions.size());
             return true;
         }
@@ -605,21 +777,9 @@ public:
         assert(ws && "Expected a non-null websocket.");
         auto tile = TileDesc::parse(tokens);
 
-        // Send back the request with all optional parameters given in the request.
-        const auto tileMsg = tile.serialize("tile:");
-#if ENABLE_DEBUG
-        const std::string response = tileMsg + " renderid=" + Util::UniqueId() + "\n";
-#else
-        const std::string response = tileMsg + "\n";
-#endif
-
-        std::vector<char> output;
-        output.reserve(response.size() + (4 * tile.getWidth() * tile.getHeight()));
-        output.resize(response.size());
-        std::memcpy(output.data(), response.data(), response.size());
-
+        size_t pixmapDataSize = 4 * tile.getWidth() * tile.getHeight();
         std::vector<unsigned char> pixmap;
-        pixmap.resize(output.capacity());
+        pixmap.resize(pixmapDataSize);
 
         std::unique_lock<std::mutex> lock(_documentMutex);
         if (!_loKitDocument)
@@ -646,18 +806,38 @@ public:
                 " ms (" << area / elapsed << " MP/s).");
         const auto mode = static_cast<LibreOfficeKitTileMode>(_loKitDocument->getTileMode());
 
-        const uint64_t hash = Png::hashBuffer(pixmap.data(), tile.getWidth(), tile.getHeight());
-        if (hash != 0 && tile.getOldHash() == hash)
+        const TileBinaryHash hash = Png::hashBuffer(pixmap.data(), tile.getWidth(), tile.getHeight());
+        TileWireId wid = _pngCache.hashToWireId(hash);
+        TileWireId oldWireId = tile.getOldWireId();
+
+        tile.setWireId(wid);
+
+        if (hash != 0 && oldWireId == wid)
         {
             // The tile content is identical to what the client already has, so skip it
-            LOG_TRC("Match oldhash==hash (" << hash << "), skipping");
+            LOG_TRC("Match oldWireId==wid (" << wid << " for hash " << hash << "); unchanged");
             return;
         }
 
-        if (!_pngCache.encodeBufferToPNG(pixmap.data(), tile.getWidth(), tile.getHeight(), output, mode, hash))
+        // Send back the request with all optional parameters given in the request.
+        std::string response = ADD_DEBUG_RENDERID(tile.serialize("tile:")) + "\n";
+
+        int pixelWidth = tile.getWidth();
+        int pixelHeight = tile.getHeight();
+
+        if (_docWatermark)
+            _docWatermark->blending(pixmap.data(), 0, 0, pixelWidth, pixelHeight, pixelWidth, pixelHeight, mode);
+
+        std::vector<char> output;
+        output.reserve(response.size() + pixmapDataSize);
+        output.resize(response.size());
+        std::memcpy(output.data(), response.data(), response.size());
+
+        if (!_pngCache.encodeBufferToPNG(pixmap.data(), tile.getWidth(), tile.getHeight(), output, mode, hash, wid, oldWireId))
         {
             //FIXME: Return error.
             //sendTextFrame("error: cmd=tile kind=failure");
+
             LOG_ERR("Failed to encode tile into PNG.");
             return;
         }
@@ -741,17 +921,29 @@ public:
             const uint64_t hash = Png::hashSubBuffer(pixmap.data(), positionX * pixelWidth, positionY * pixelHeight,
                                                      pixelWidth, pixelHeight, pixmapWidth, pixmapHeight);
 
-            if (hash != 0 && tiles[tileIndex].getOldHash() == hash)
+            TileWireId wireId = _pngCache.hashToWireId(hash);
+            TileWireId oldWireId = tiles[tileIndex].getOldWireId();
+            if (hash != 0 && oldWireId == wireId)
             {
                 // The tile content is identical to what the client already has, so skip it
                 LOG_TRC("Match for tile #" << tileIndex << " at (" << positionX << "," <<
-                        positionY << ") oldhash==hash (" << hash << "), skipping");
+                        positionY << ") oldhash==hash (" << hash << "), wireId: " << wireId << " skipping");
                 tiles.erase(tiles.begin() + tileIndex);
                 continue;
             }
 
+            int offsetX = positionX  * pixelWidth;
+            int offsetY = positionY * pixelHeight;
+
+            if (_docWatermark)
+                _docWatermark->blending(pixmap.data(), offsetX, offsetY,
+                                        pixmapWidth, pixmapHeight,
+                                        tileCombined.getWidth(), tileCombined.getHeight(),
+                                        mode);
+
             if (!_pngCache.encodeSubBufferToPNG(pixmap.data(), positionX * pixelWidth, positionY * pixelHeight,
-                                                pixelWidth, pixelHeight, pixmapWidth, pixmapHeight, output, mode, hash))
+                                                pixelWidth, pixelHeight, pixmapWidth, pixmapHeight, output, mode,
+                                                hash, wireId, oldWireId))
             {
                 //FIXME: Return error.
                 //sendTextFrame("error: cmd=tile kind=failure");
@@ -760,9 +952,9 @@ public:
             }
 
             const auto imgSize = output.size() - oldSize;
-            LOG_TRC("Encoded tile #" << tileIndex << " at (" << positionX << "," << positionY << ") with oldhash=" <<
-                    tiles[tileIndex].getOldHash() << ", hash=" << hash << " in " << imgSize << " bytes.");
-            tiles[tileIndex].setHash(hash);
+            LOG_TRC("Encoded tile #" << tileIndex << " at (" << positionX << "," << positionY << ") with oldWireId=" <<
+                    tiles[tileIndex].getOldWireId() << ", hash=" << hash << " wireId: " << wireId << " in " << imgSize << " bytes.");
+            tiles[tileIndex].setWireId(wireId);
             tiles[tileIndex].setImgSize(imgSize);
             tileIndex++;
         }
@@ -772,11 +964,7 @@ public:
                 renderArea.getWidth() << ", " << renderArea.getHeight() << ") " <<
                 " took " << (elapsed/1000.) << " ms (including the paintTile).");
 
-#if ENABLE_DEBUG
-        const auto tileMsg = tileCombined.serialize("tilecombine:") + " renderid=" + Util::UniqueId() + "\n";
-#else
-        const auto tileMsg = tileCombined.serialize("tilecombine:") + "\n";
-#endif
+        const auto tileMsg = ADD_DEBUG_RENDERID(tileCombined.serialize("tilecombine:")) + "\n";
         LOG_TRC("Sending back painted tiles for " << tileMsg);
 
         std::vector<char> response;
@@ -925,7 +1113,8 @@ private:
                 const std::string& docPassword,
                 const std::string& renderOpts,
                 const bool haveDocPassword,
-                const std::string& lang) override
+                const std::string& lang,
+                const std::string& watermarkText) override
     {
         std::unique_lock<std::mutex> lock(_mutex);
 
@@ -954,7 +1143,7 @@ private:
 
         try
         {
-            if (!load(session, uri, userName, docPassword, renderOpts, haveDocPassword, lang))
+            if (!load(session, uri, userName, docPassword, renderOpts, haveDocPassword, lang, watermarkText))
             {
                 return false;
             }
@@ -1045,6 +1234,11 @@ private:
         return _tileQueue;
     }
 
+    int getEditorId() override
+    {
+        return _editorId;
+    }
+
     /// Notify all views of viewId and their associated usernames
     void notifyViewInfo() override
     {
@@ -1075,10 +1269,12 @@ private:
             }
             else
             {
-                oss << "\"userid\":\"" << itView->second.userid << "\",";
-                const auto username = itView->second.username;
+                oss << "\"userid\":\"" << itView->second.UserId << "\",";
+                const auto username = itView->second.Username;
                 oss << "\"username\":\"" << username << "\",";
-                const auto readonly = itView->second.isReadOnly;
+                if (!itView->second.UserExtraInfo.empty())
+                    oss << "\"userextrainfo\":" << itView->second.UserExtraInfo << ",";
+                const auto readonly = itView->second.IsReadOnly;
                 oss << "\"readonly\":\"" << readonly << "\",";
                 const auto it = viewColorsMap.find(username);
                 if (it != viewColorsMap.end())
@@ -1098,6 +1294,51 @@ private:
         sendTextFrame("client-all " + msg);
     }
 
+    void updateEditorSpeeds(int id, int speed) override
+    {
+        int maxSpeed = -1, fastestUser = -1;
+
+        auto now = std::chrono::steady_clock::now();
+        _lastUpdatedAt[id] = now;
+        _speedCount[id] = speed;
+
+        for (const auto& it : _sessions)
+        {
+            const auto session = it.second;
+            int sessionId = session->getViewId();
+
+            auto duration = (_lastUpdatedAt[id] - now);
+            auto durationInMs = std::chrono::duration_cast<std::chrono::milliseconds>(duration).count();
+            if (_speedCount[sessionId] != 0 && durationInMs > 5000)
+            {
+                _speedCount[sessionId] = session->getSpeed();
+                _lastUpdatedAt[sessionId] = now;
+            }
+            if (_speedCount[sessionId] > maxSpeed)
+            {
+                maxSpeed = _speedCount[sessionId];
+                fastestUser = sessionId;
+            }
+        }
+        // 0 for preventing selection of the first always
+        // 1 for preventing the new users from directly beoming the editors
+        if (_editorId != fastestUser && (maxSpeed != 0 && maxSpeed != 1)) {
+            if (!_editorChangeWarning && _editorId != -1)
+            {
+                _editorChangeWarning = true;
+            }
+            else
+            {
+                _editorChangeWarning = false;
+                _editorId = fastestUser;
+                for (const auto& it : _sessions)
+                    it.second->sendTextFrame("editor: " + std::to_string(_editorId));
+            }
+        }
+        else
+            _editorChangeWarning = false;
+    }
+
 private:
 
     // Get the color value for all author names from the core
@@ -1144,7 +1385,8 @@ private:
                                         const std::string& docPassword,
                                         const std::string& renderOpts,
                                         const bool haveDocPassword,
-                                        const std::string& lang)
+                                        const std::string& lang,
+                                        const std::string& watermarkText)
     {
         const std::string sessionId = session->getId();
 
@@ -1209,6 +1451,9 @@ private:
             // Only save the options on opening the document.
             // No support for changing them after opening a document.
             _renderOpts = renderOpts;
+
+            if (!watermarkText.empty())
+                _docWatermark.reset(new Watermark(_loKitDocument, watermarkText));
         }
         else
         {
@@ -1249,7 +1494,8 @@ private:
 
         const int viewId = _loKitDocument->getView();
         session->setViewId(viewId);
-        _sessionUserInfo[viewId] = UserInfo({session->getViewUserId(), session->getViewUserName(), session->isReadOnly()});
+        _sessionUserInfo[viewId] = UserInfo(session->getViewUserId(), session->getViewUserName(),
+                                            session->getViewUserExtraInfo(), session->isReadOnly());
 
         _viewIdToCallbackDescr.emplace(viewId,
                                        std::unique_ptr<CallbackDescriptor>(new CallbackDescriptor({ this, viewId })));
@@ -1295,6 +1541,9 @@ private:
                 if (size == disconnect.size() &&
                     strncmp(data, disconnect.data(), disconnect.size()) == 0)
                 {
+                    if(session->getViewId() == _editorId) {
+                        _editorId = -1;
+                    }
                     LOG_DBG("Removing ChildSession [" << sessionId << "].");
                     _sessions.erase(it);
                     const auto count = _sessions.size();
@@ -1315,7 +1564,7 @@ private:
                     vect.assign(data, data + size);
 
                     // TODO loolnb - this is probably wrong...
-                    session->handleMessage(/* fin = */ false, WebSocketHandler::WSOpCode::Binary, vect);
+                    session->handleMessage(/* fin = */ false, WSOpCode::Binary, vect);
                     return true;
                 }
             }
@@ -1375,7 +1624,7 @@ private:
 
         LOG_DBG("Thread started.");
 
-        // Update memory stats every 5 seconds.
+        // Update memory stats and editor every 5 seconds.
         const auto memStatsPeriodMs = 5000;
         auto lastMemStatsTime = std::chrono::steady_clock::now();
         sendTextFrame(Util::getMemoryStats(ProcSMapsFile));
@@ -1387,14 +1636,13 @@ private:
                 const TileQueue::Payload input = _tileQueue->get(POLL_TIMEOUT_MS * 2);
                 if (input.empty())
                 {
-                    const auto duration = (std::chrono::steady_clock::now() - lastMemStatsTime);
-                    const auto durationMs = std::chrono::duration_cast<std::chrono::milliseconds>(duration).count();
+                    auto duration = (std::chrono::steady_clock::now() - lastMemStatsTime);
+                    auto durationMs = std::chrono::duration_cast<std::chrono::milliseconds>(duration).count();
                     if (durationMs > memStatsPeriodMs)
                     {
                         sendTextFrame(Util::getMemoryStats(ProcSMapsFile));
                         lastMemStatsTime = std::chrono::steady_clock::now();
                     }
-
                     continue;
                 }
 
@@ -1555,6 +1803,9 @@ private:
     // Whether password is required to view the document, or modify it
     PasswordType _docPasswordType;
 
+    // Document watermark
+    std::unique_ptr<Watermark> _docWatermark;
+
     std::atomic<bool> _stop;
     mutable std::mutex _mutex;
 
@@ -1564,9 +1815,13 @@ private:
 
     std::condition_variable _cvLoading;
     std::atomic_size_t _isLoading;
+    int _editorId;
+    bool _editorChangeWarning;
     std::map<int, std::unique_ptr<CallbackDescriptor>> _viewIdToCallbackDescr;
     std::map<std::string, std::shared_ptr<ChildSession>> _sessions;
 
+    std::map<int, std::chrono::steady_clock::time_point> _lastUpdatedAt;
+    std::map<int, int> _speedCount;
     /// For showing disconnected user info in the doc repair dialog.
     std::map<int, UserInfo> _sessionUserInfo;
     Poco::Thread _callbackThread;
@@ -1766,6 +2021,27 @@ void lokit_main(const std::string& childRoot,
             std::_Exit(Application::EXIT_SOFTWARE);
         }
 
+        rlimit rlim = { 0, 0 };
+        if (getrlimit(RLIMIT_AS, &rlim) == 0)
+            LOG_INF("RLIMIT_AS is " << rlim.rlim_max << " bytes.");
+        else
+            LOG_SYS("Failed to get RLIMIT_AS.");
+
+        if (getrlimit(RLIMIT_STACK, &rlim) == 0)
+            LOG_INF("RLIMIT_STACK is " << rlim.rlim_max << " bytes.");
+        else
+            LOG_SYS("Failed to get RLIMIT_STACK.");
+
+        if (getrlimit(RLIMIT_FSIZE, &rlim) == 0)
+            LOG_INF("RLIMIT_FSIZE is " << rlim.rlim_max << " bytes.");
+        else
+            LOG_SYS("Failed to get RLIMIT_FSIZE.");
+
+        if (getrlimit(RLIMIT_NOFILE, &rlim) == 0)
+            LOG_INF("RLIMIT_NOFILE is " << rlim.rlim_max << " files.");
+        else
+            LOG_SYS("Failed to get RLIMIT_NOFILE.");
+
         assert(loKit);
         LOG_INF("Process is ready.");
 
@@ -1810,7 +2086,7 @@ void lokit_main(const std::string& childRoot,
 #endif
 
                     LOG_DBG(socketName << ": recv [" << LOOLProtocol::getAbbreviatedMessage(message) << "].");
-                    StringTokenizer tokens(message, " ", StringTokenizer::TOK_IGNORE_EMPTY | StringTokenizer::TOK_TRIM);
+                    std::vector<std::string> tokens = LOOLProtocol::tokenize(message);
 
                     // Note: Syntax or parsing errors here are unexpected and fatal.
                     if (TerminationFlag)
@@ -1856,6 +2132,14 @@ void lokit_main(const std::string& childRoot,
                             LOG_WRN("No document while processing " << tokens[0] << " request.");
                         }
                     }
+                    else if (tokens.size() == 3 && tokens[0] == "setconfig")
+                    {
+                        // Currently onlly rlimit entries are supported.
+                        if (!Rlimit::handleSetrlimitCommand(tokens))
+                        {
+                            LOG_ERR("Unknown setconfig command: " << message);
+                        }
+                    }
                     else
                     {
                         LOG_ERR("Bad or unknown token [" << tokens[0] << "]");
