@@ -73,7 +73,7 @@ L.Map.include({
 
 		this.showBusy(_('Downloading...'), false);
 		this._socket.sendMessage('downloadas ' +
-			'name=' + name + ' ' +
+			'name=' + encodeURIComponent(name) + ' ' +
 			'id=' + id + ' ' +
 			'format=' + format + ' ' +
 			'options=' + options);
@@ -85,6 +85,9 @@ L.Map.include({
 	},
 
 	saveAs: function (url, format, options) {
+		if (url === undefined || url == null) {
+			return;
+		}
 		if (format === undefined || format === null) {
 			format = '';
 		}
@@ -93,13 +96,10 @@ L.Map.include({
 		}
 
 		this.showBusy(_('Saving...'), false);
-		// TakeOwnership: we are performing a 'real' save-as, the document
-		// is just getting a new place, ie. it will get the
-		// '.uno:ModifiedStatus' upon completion.
 		this._socket.sendMessage('saveas ' +
-			'url=' + url + ' ' +
+			'url=wopi:' + encodeURIComponent(url) + ' ' +
 			'format=' + format + ' ' +
-			'options=TakeOwnership,' + options);
+			'options=' + options);
 	},
 
 	applyStyle: function (style, familyName) {
@@ -131,7 +131,9 @@ L.Map.include({
 	},
 
 	save: function(dontTerminateEdit, dontSaveIfUnmodified) {
-		this._socket.sendMessage('save dontTerminateEdit=' + (dontTerminateEdit ? 1 : 0) + ' dontSaveIfUnmodified=' + (dontSaveIfUnmodified ? 1 : 0));
+		this._socket.sendMessage('save' +
+		                         ' dontTerminateEdit=' + (dontTerminateEdit ? 1 : 0) +
+		                         ' dontSaveIfUnmodified=' + (dontSaveIfUnmodified ? 1 : 0));
 	},
 
 	sendUnoCommand: function (command, json) {
@@ -174,7 +176,7 @@ L.Map.include({
 
 	showLOKeyboardHelp: function() {
 		var w = window.innerWidth / 2;
-		$.get('/loleaflet/dist/loleaflet-help.html', function(data) {
+		$.get('loleaflet-help.html', function(data) {
 			vex.open({
 				content: data,
 				showCloseButton: true,
@@ -208,6 +210,10 @@ L.Map.include({
 					for (i = 0, max = translatableContent.length; i < max; i++) {
 						translatableContent[i].firstChild.nodeValue = translatableContent[i].firstChild.nodeValue.toLocaleString();
 					}
+					translatableContent = $vexContent.find('p');
+					for (i = 0, max = translatableContent.length; i < max; i++) {
+						translatableContent[i].firstChild.nodeValue = translatableContent[i].firstChild.nodeValue.toLocaleString();
+					}
 
 					$('.vex-content').attr('tabindex', -1);
 					$('.vex-content').focus();
